-- prepared by Steve Stedman
-- http://SteveStedman.com
-- twitter:  @SqlEmt
-- Date/Time Functions

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG;

------------------------ EXAMPLES SETUP ------------------------

USE Master;
GO

IF EXISTS(SELECT name 
            FROM sys.databases 
		   WHERE name = 'QueryTraining')
BEGIN
	ALTER DATABASE [QueryTraining] 
	  SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [QueryTraining];
END
CREATE DATABASE [QueryTraining];
GO

USE [QueryTraining];
GO


------------------------------------------------
-- GETDATE()
SELECT GETDATE();

-- GETDATE just as a TIME
SELECT CONVERT (time, GETDATE());

-- comparing GETDATE() to others
SELECT CONVERT (time, GETDATE()) AS [getDate],
       CONVERT (time, SYSDATETIME()) AS [sysDateTime],
       CONVERT (time, SYSUTCDATETIME()) AS [sysUtcDateTime],
       CONVERT (time, CURRENT_TIMESTAMP) AS [current_timestamp],
       CONVERT (time, GETUTCDATE()) AS [getUtcDate];



------------------------------------------------
-- DATEPART()
 SELECT DATEPART(year, GETDATE()) AS [year],
		DATEPART(quarter, GETDATE()) AS [quarter],
		DATEPART(month, GETDATE()) AS [month],
		DATEPART(dayofyear, GETDATE()) AS [dayofyear],
		DATEPART(day, GETDATE()) AS [day],
		DATEPART(week, GETDATE()) AS [week],
		DATEPART(weekday, GETDATE()) AS [weekday],
		DATEPART(hour, GETDATE()) AS [hour],
		DATEPART(minute, GETDATE()) AS [minute],
		DATEPART(second, GETDATE()) AS [second],
		DATEPART(millisecond, GETDATE()) AS [millisecond],
		DATEPART(microsecond, GETDATE()) AS [microsecond],
		DATEPART(nanosecond, GETDATE()) AS [nanosecond],
		DATEPART(ISO_WEEK, GETDATE()) AS [ISO_WEEK];

------------------------------------------------
-- DATEADD()
SELECT DATEADD(month, 1, GETDATE());

------------------------------------------------
-- DATEDIFF()
SELECT DATEDIFF(week, '1/1/2015', GETDATE());
-- does the order matter?
SELECT DATEDIFF(week, GETDATE(), '1/1/2015');



SELECT DATEDIFF(year, '1/1/2015', GETDATE());
SELECT DATEDIFF(quarter, '1/1/2015', GETDATE());
SELECT DATEDIFF(month, '1/1/2015', GETDATE());
SELECT DATEDIFF(dayofyear, '1/1/2015', GETDATE());
SELECT DATEDIFF(day, '1/1/2015', GETDATE());
SELECT DATEDIFF(week, '1/1/2015', GETDATE());
SELECT DATEDIFF(hour, '1/1/2015', GETDATE());
SELECT DATEDIFF(minute, '1/1/2015', GETDATE());
SELECT DATEDIFF(second, '1/1/2015', GETDATE());
SELECT DATEDIFF(millisecond, '1/1/2015', GETDATE());
SELECT DATEDIFF(microsecond, '1/1/2015', GETDATE());


-- to help eliminate the overflow.
SELECT DATEDIFF(millisecond, '3/19/2015', GETDATE());
SELECT DATEDIFF(microsecond, '3/19/2015', GETDATE());


SELECT DATEDIFF(microsecond, '3/19/2015 11:00am', GETDATE());




-- DATEPART, DATEADD, DATEDIFF used for rounding....
-- http://stevestedman.com/2012/10/tsql-rounding-datetime-to-nearest-seconds/
declare @dtVariable as DATETIME;
set @dtVariable = GETDATE();
 
-- if you are trying this before noon, uncomment the
--     following line to simulate an after noon time.
--set @dtVariable = dateadd(hour, 12, @dtVariable)
 
-- Rounding to the second
select @dtVariable as Original,
       DATEADD(ms, 500 - DATEPART(ms, @dtVariable + '00:00:00.500'),
               @dtVariable) as RoundedToSecond;
 
-- Truncated to the minute
select @dtVariable as Original,
       DATEADD(minute, DATEDIFF(minute, 0, @dtVariable), 0) as TruncatedToMinute;
 
-- Rounded to minute
select @dtVariable as Original,
       DATEADD(minute, DATEDIFF(minute, 0,
             DATEADD(second, 30 - DATEPART(second, @dtVariable + '00:00:30.000'),
             @dtVariable)), 0) as RoundedToMinute;
 
-- Truncated to the hour
select @dtVariable as Original,
       DATEADD(hour, DATEDIFF(hour, 0, @dtVariable), 0) as TruncatedToHour;
 
-- Rounded to hour
select @dtVariable as Original,
       DATEADD(hour, DATEDIFF(hour, 0,
             DATEADD(minute, 30 - DATEPART(minute, @dtVariable + '00:30:00.000'),
             @dtVariable)), 0)  as RoundedToHour;
 
-- Truncated to the day
select @dtVariable as Original,
       DATEADD(Day, DATEDIFF(Day, 0, @dtVariable), 0) as TruncatedToDay;
 
-- Rounded to day
select @dtVariable as Original,
       DATEADD(day, DATEDIFF(day, 0,
             DATEADD(hour, 12 - DATEPART(hour, @dtVariable + '12:00:00.000'),
             @dtVariable)), 0)  as RoundedToDay;






------------------------------------------------
-- DATEFROMPARTS (2012)

-- the old way
SELECT Date=cast(Convert(datetime,convert(varchar(10),2012)+'-'+convert(varchar(10),6)+'-'+convert(varchar(10),1), 101)as date)

-- DATEFROMPARTS(year,month,day)
SELECT DateFROMParts ( 2012, 11, 10 ) AS Date;

-- Do we need the 4 digit year??
SELECT DateFROMParts ( 12, 11, 10 ) AS Date;


------------------------------------------------
-- TIMEFROMPARTS (2012)

--TIMEFROMPARTS (hour,minute,seconds,fractions,precision)
SELECT TimeFROMParts(11,15,20,147,3)



------------------------------------------------
-- DATETIMEFROMPARTS (2012)

--DATETIMEFROMPARTS (year,month,day,hour,minute,seconds,milliseconds)
SELECT  DateTimeFROMParts(2012, 11, 10, 11, 15, 49, 147)



------------------------------------------------
-- DATETIME2FROMPARTS (2012)

--DATETIME2FROMPARTS (year,month,day,hour,minute,seconds,fractions,precision)
SELECT DateTime2FROMParts(2012, 11, 10, 11, 15, 49, 147, 4);

-- this will throw an error
--   the precision is smaller than the fraction specified
SELECT DateTime2FROMParts(2012, 11, 10, 11, 15, 49, 147, 1);

------------------------------------------------
-- SMALLDATETIMEFROMPARTS (2012)

--SMALLDATETIMEFROMPARTS(year,month,day,hour,minute)
SELECT SmallDateTimeFROMParts(2012, 11, 10, 11, 15);



------------------------------------------------
-- DATETIMEOFFSETFROMPARTS 

--DATETIMEOFFSETFROMPARTS ( year, month, day, hour, minute, seconds, fractions, hour_offset, minute_offset, precision )
SELECT DateTimeOffsetFROMParts (2012, 11, 10, 11, 15, 25, 120, 8, 0, 3);




------------------------------------------------
-- EOMONTH (2012)
DECLARE @date DATETIME
SET @date = DATEFROMPARTS(2012, 11, 10)
SELECT EOMONTH ( @date ) AS Result;

-- next month end of month
SELECT EOMONTH ( @date, +1) AS Result;

-- last month end of month
SELECT EOMONTH ( @date, -1) AS Result;


